from __future__ import (absolute_import, unicode_literals)

from collections import OrderedDict
from copy import deepcopy
import importlib

from django.conf import settings
from django.core import urlresolvers
from django.core.exceptions import FieldDoesNotExist, ImproperlyConfigured
from django.core.urlresolvers import NoReverseMatch

from . import defaults


def is_active(path, path_to_check):
    if not path_to_check:
        return False
    return path == path_to_check

# TODO: menu needs to hide entries not available to a certain user role
# by getting the view class from the named url we can check which permissions
# are available:
# http://stackoverflow.com/questions/5749075


def menu(menu_config=None, **kwargs):
    """
    Tranforms a menu definition into a dictionary which is a frendlier format
    to parse in a template.
    """
    request = kwargs.pop('request', None)
    user = kwargs.pop('user', None)

    if not menu_config:
        menu_config = settings.ARCTIC_MENU

    menu_dict = OrderedDict()
    for menu_entry in menu_config:
        if type(menu_entry) in (list, tuple):

            # check permission based on named_url
            path = None
            if menu_entry[1]:
                if not view_from_url(menu_entry[1]).has_permission(user):
                    continue

                path = urlresolvers.reverse(menu_entry[1])
            # icons and collapse are optional
            icon = None
            if (len(menu_entry) >= 3) and \
               (not type(menu_entry[2]) in (list, tuple)):
                icon = menu_entry[2]
            active_weight = len(path) if path else 0
            menu_dict[menu_entry[0]] = {
                'url': menu_entry[1],
                'icon': icon,
                'submenu': None,
                'active': is_active(request.path, path),
                'active_weight': active_weight
            }

            # check if the last item in a menu entry is a submenu
            last_item = menu_entry[-1]
            if type(last_item) in (list, tuple):
                menu_dict[menu_entry[0]]['submenu'] = menu(last_item,
                                                           user=user,
                                                           request=request)
    return menu_clean(menu_dict)


def menu_clean(menu_config):
    """
    Make sure that only the menu item with the largest weight is active.
    If a child of a menu item is active, the parent should be active too.
    :param menu:
    :return:
    """
    max_weight = -1
    for _, value in list(menu_config.items()):
        if value['submenu']:
            for _, v in list(value['submenu'].items()):
                if v['active']:
                    # parent inherits the weight of the axctive child
                    value['active'] = True
                    value['active_weight'] = v['active_weight']

        if value['active']:
            max_weight = max(value['active_weight'], max_weight)

    if max_weight > 0:
        # one of the items is active: make items with lesser weight inactive
        for _, value in list(menu_config.items()):
            if value['active'] and value['active_weight'] < max_weight:
                max_weight = max(value['active_weight'], max_weight)
                value['active'] = False
    return menu_config


def view_from_url(named_url):  # noqa
    """
    Finds and returns the view class from a named url
    """
    # code below is `stolen` from django's reverse method
    resolver = urlresolvers.get_resolver(urlresolvers.get_urlconf())

    if type(named_url) in (list, tuple):
        named_url = named_url[0]
    parts = named_url.split(':')
    parts.reverse()
    view = parts[0]
    path = parts[1:]
    current_path = None
    resolved_path = []
    ns_pattern = ''

    while path:
        ns = path.pop()
        current_ns = current_path.pop() if current_path else None

        # Lookup the name to see if it could be an app identifier
        try:
            app_list = resolver.app_dict[ns]
            # Yes! Path part matches an app in the current Resolver
            if current_ns and current_ns in app_list:
                # If we are reversing for a particular app,
                # use that namespace
                ns = current_ns
            elif ns not in app_list:
                # The name isn't shared by one of the instances
                # (i.e., the default) so just pick the first instance
                # as the default.
                ns = app_list[0]
        except KeyError:
            pass

        if ns != current_ns:
            current_path = None

        try:
            extra, resolver = resolver.namespace_dict[ns]
            resolved_path.append(ns)
            ns_pattern = ns_pattern + extra
        except KeyError as key:
            if resolved_path:
                raise NoReverseMatch(
                    "%s is not a registered namespace inside '%s'" %
                    (key, ':'.join(resolved_path)))
            else:
                raise NoReverseMatch("%s is not a registered namespace" %
                                     key)
    if ns_pattern:
        resolver = urlresolvers.get_ns_resolver(ns_pattern, resolver)

    # custom code, get view from reverse_dict
    reverse_dict = resolver.reverse_dict.dict()
    for key, url_obj in reverse_dict.items():
        if url_obj == reverse_dict[view] \
                and key != view:
            module = importlib.import_module(key.__module__)
            return getattr(module, key.__name__)


def find_attribute(obj, value):
    """
    Finds the attribute connected to the last object when a chain of
    connected objects is given in a string separated with double underscores.
    For example when a model x has a foreign key to model y and model y has
    attribute a, findattr(x, 'y__a') will return the a attribute from the y
    model that exists in x.
    """
    if '__' in value:
        value_list = value.split('__')
        attr = get_attribute(obj, value_list[0])
        return find_attribute(attr, '__'.join(value_list[1:]))
    return get_attribute(obj, value)


def get_attribute(obj, value):
    """
    Normally the result of list_items for listviews are a set of model objects.
    But when you want a GROUP_BY query (with 'values' method), than
    the result will be a dict. This method will help you find an item for
    either objects or dictionaries.
    """
    if type(obj) == dict:
        return dict.get(obj, value)
    else:
        return getattr(obj, value)


def find_field_meta(obj, value):
    """
    In a model, finds the attribute meta connected to the last object when
    a chain of  connected objects is given in a string separated with double
    underscores.
    """
    if '__' in value:
        value_list = value.split('__')
        child_obj = obj._meta.get_field(value_list[0]).rel.to
        return find_field_meta(child_obj, '__'.join(value_list[1:]))
    return obj._meta.get_field(value)


def get_field_class(qs, field_name):
    """
    Given a queryset and a field name, it will return the field's class
    """
    try:
        return qs.model._meta.get_field(field_name).__class__.__name__
    # while annotating, it's possible that field does not exists.
    except FieldDoesNotExist:
        return None


def arctic_setting(setting_name, valid_options=None):
    """
    Tries to get a setting from the django settings, if not available defaults
    to the one defined in defaults.py
    """
    try:
        value = getattr(settings, setting_name)
        if valid_options and value not in valid_options:
            error_message = 'Invalid value for {}, must be one of: {}'.format(
                setting_name, str(valid_options))
            raise ImproperlyConfigured(error_message)
    except AttributeError:
        pass
    return getattr(settings, setting_name, getattr(defaults, setting_name))


class RemoteDataSet():
    url_template = '?{filters}{fields}{order}{paginate}'
    paginate_template = '&offset={}&limit={}'
    order_separator = ','
    order_template = '&order={}'
    fields_separator = ','
    fields_template = '&fields={}'
    filters_template = '{}'
    filters_template_kv = '&{key}={value}'
    count = -1
    _options = {'fields': '',
                'order': '',
                'filters': '',
                'paginate': ''}
    page_size = None

    def fields(self, fields):
        if fields:
            fields_str = self.fields_separator.join(fields)
            self._options['fields'] = self.fields_template.format(fields_str)
        return deepcopy(self)

    def order_by(self, order):
        if order:
            order_str = self.order_separator.join(order)
            self._options['order'] = self.order_template.format(order_str)
        return deepcopy(self)

    def filter(self, **kwargs):
        if kwargs:
            filters = ''
            for key, value in kwargs.items():
                filters += self.filters_template_kv.format(
                    key=key,
                    value=value)
            self._options['filters'] = self.filters_template.format(filters)
        return deepcopy(self)

    def get_url(self, start, stop):
        self._options['paginate'] = self.paginate_template.format(start, stop)
        url = self.url_template.format(**self._options)
        return url.replace('?&', '?')

    def get(self, start, stop):
        pass

    def __len__(self):
        return self.count

    def __getitem__(self, slice):
        return self.get(slice.start, slice.stop)


def offset_limit(func):
    """
    Decorator that converts python slicing to offset and limit
    """
    def func_wrapper(self, start, stop):
        offset = start
        limit = stop - start
        return func(self, offset, limit)
    return func_wrapper



from __future__ import (absolute_import, unicode_literals)

from django import forms

from .models import Article
from django.db.models import Q


class ArticleForm(forms.ModelForm):
    class Meta:
        fields = ['title', 'description', 'category', 'updated_at', 'tags',
                  'published']
        model = Article

    def __init__(self, *args, **kwargs):
        super(ArticleForm, self).__init__(*args, **kwargs)
        self.fields['category'].empty_label = ""


class AdvancedArticleSearchForm(forms.Form):
    description = forms.CharField(max_length=100,
                                  required=False,
                                  label='Description')

    def __init__(self, data):
        # Reset data, but store for get_search_filter
        self.stored_data = data
        super(AdvancedArticleSearchForm, self).__init__()

    def get_search_filter(self):
        value = self.stored_data.get('description')
        if value:
            return Q(description__icontains=value)
        return Q()



ARCTIC_SITE_NAME = 'Arctic Example'

# Menu format:
# (('menu label', 'named url', 'optional icon class', (optional submenu)))
ARCTIC_MENU = (
    ('Dashboard', 'index', 'fa-dashboard'),
    ('Articles', None, 'fa-file-text-o', (
        ('List', 'articles:list'),
        ('Create', 'articles:create'),
    )),
    ('Categories', None, 'fa-sitemap', (
        ('List', 'articles:category-list'),
        ('Create', 'articles:category-create'),
    )),
    ('Tags', None, 'fa-tags', (
        ('List', 'articles:tag-list'),
        ('Create', 'articles:tag-create'),
    )),
    ('Users', None, 'fa-user', (
        ('List', 'users:list'),
        ('Create', 'users:create'),
    )),
    ('Countries', 'countries-list', 'fa-globe'),
)

ARCTIC_USER_ROLE_MODEL = 'arctic.UserRole'
ARCTIC_ROLE_MODEL = 'arctic.Role'

ARCTIC_ROLES = {
    'editor': (
        'view_dashboard',
        'view_article',
        'change_article',
        'add_article',
    ),
}

ARCTIC_AUTOCOMPLETE = {
    'categories': ('articles.Category', 'name')
}

# ARCTIC_SIDEBAR_BACKGROUND = '#BF1D1D'
# ARCTIC_HIGHLIGHT_COLOR = '#9ec3d5'

# ARCTIC_SIDEBAR_BACKGROUND = '#212050'
# ARCTIC_HIGHLIGHT_COLOR = '#76a2cb'



from datetime import datetime

from django.forms.widgets import (DateInput, DateTimeInput, Select,
                                  SelectMultiple, TimeInput)
from django.utils.safestring import mark_safe


class StyledSelect(Select):
    def render(self, name, value, attrs=None, renderer=None):
        try:
            select_render = super(StyledSelect, self).render(
                name, value, attrs, renderer)
        except TypeError:  # versions older than Django 1.11
            select_render = super(StyledSelect, self).render(
                name, value, attrs)

        return mark_safe('<div class="styled-select">{}</div>'.format(
            select_render))


class Selectize(Select):
    def __init__(self, attrs, choices):
        attrs['js-selectize'] = True
        super(Selectize, self).__init__(attrs, choices)


class SelectizeMultiple(SelectMultiple):
    def __init__(self, attrs, choices):
        attrs['js-selectize-multiple'] = True
        super(SelectizeMultiple, self).__init__(attrs, choices)


class SelectizeAutoComplete(Select):
    def __init__(self, attrs, choices, url):
        attrs['js-selectize-autocomplete'] = True
        attrs['data-url'] = url
        super(SelectizeAutoComplete, self).__init__(attrs, choices)


class DateTimePickerInput(DateTimeInput):
    def __init__(self, attrs, format):
        attrs['js-datetimepicker'] = True
        super(DateTimePickerInput, self).__init__(attrs, format)

    def get_context(self, name, value, attrs):
        context = super(DateTimePickerInput, self).get_context(name, value,
                                                               attrs)
        if not value:
            value = datetime.now()
        context['widget']['attrs']['data-datetime'] =\
            value.strftime('%m/%d/%Y %I:%M %p')
        return context


class DatePickerInput(DateInput):
    def __init__(self, attrs, format):
        attrs['js-datepicker'] = True
        super(DatePickerInput, self).__init__(attrs, format)

    def get_context(self, name, value, attrs):
        context = super(DatePickerInput, self).get_context(name, value, attrs)
        if not value:
            value = datetime.now()
        context['widget']['attrs']['data-date'] =\
            value.strftime('%m/%d/%Y')
        return context


class TimePickerInput(TimeInput):
    def __init__(self, attrs, format):
        attrs['js-timepicker'] = True
        super(TimePickerInput, self).__init__(attrs, format)

    def get_context(self, name, value, attrs):
        context = super(TimePickerInput, self).get_context(name, value, attrs)
        if not value:
            value = datetime.now()
        context['widget']['attrs']['data-time'] =\
            value.strftime('%I:%M %p')
        return context



# -*-*- encoding: utf-8 -*-*-
from __future__ import (absolute_import, unicode_literals)

from django.conf import settings
from django.conf.urls import (include, url)
from django.conf.urls.static import static

from arctic.generics import LoginView
from arctic.urls import autocomplete_url
from arctic.users.urls import user_patterns
from arctic.views import (handler400, handler403, handler404, handler500)  # noqa

from countries.views import (CountryAPIView, CountryListView)
from dashboard.views import DashboardView

urlpatterns = [
    url(r'^$', DashboardView.as_view(), name='index'),
    url(r'^login/$', LoginView.as_view(), name='login'),
    url(r'^articles/', include('articles.urls', 'articles')),
    url(r'^users/', include(user_patterns, namespace='users')),
    url(r'^countries/$', CountryListView.as_view(), name='countries-list'),
    url(r'^countries-api/$', CountryAPIView.as_view(), name='countries-api'),
    autocomplete_url,
]


if settings.DEBUG:
    try:
        import debug_toolbar
        urlpatterns = [
            url(r'^__debug__/', include(debug_toolbar.urls)),
        ] + urlpatterns
    except ImportError:
        pass

    urlpatterns += static(settings.MEDIA_URL,
                          document_root=settings.MEDIA_ROOT)



from __future__ import (absolute_import, unicode_literals)

from django.core.urlresolvers import (reverse, reverse_lazy)
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

from arctic.generics import (CreateView, DeleteView, ListView, TemplateView,
                             UpdateView)
from collections import OrderedDict

from .forms import ArticleForm, AdvancedArticleSearchForm
from .inlines import TagsInline
from .models import (Article, Category, Tag)


class DashboardView(TemplateView):
    template_name = 'arctic/index.html'
    page_title = 'Dashboard'
    permission_required = 'view_dashboard'

    def get_context_data(self, **kwargs):
        context = super(DashboardView, self).get_context_data(**kwargs)
        return context


class ArticleListView(ListView):
    paginate_by = 2
    model = Article
    fields = ['title', 'description', 'published', 'category', 'tags']
    ordering_fields = ['title', 'description', 'published']
    search_fields = ['title']
    advanced_search_form = AdvancedArticleSearchForm
    breadcrumbs = (('Home', 'index'), ('Article List', None))
    action_links = [
        ('delete', 'articles:delete', 'fa-trash'),
    ]
    field_links = {
        'title': 'articles:detail',
        'category': ('articles:category-detail', 'category_id'),
    }
    tool_links = [
        (_('Create Article'), 'articles:create', 'fa-plus'),
    ]

    permission_required = "view_article"

    def get_category_field(self, row):
        return row.category.name

    def get_published_field(self, row):
        symbol = 'fa-check' if row.published else 'fa-minus'
        return mark_safe('<i class="fa {}"></i>'.format(symbol))


class ArticleUpdateView(UpdateView):
    page_title = _("Edit Article")
    permission_required = "change_article"
    model = Article
    success_url = reverse_lazy('articles:list')
    inlines = [TagsInline]
    form_class = ArticleForm
    links = [
        ('Back to list', 'articles:list'),
    ]
    layout = OrderedDict([
                        ('',
                         ['title', ['category', 'tags|5']]),
                        ('Body|Extra Information for this fieldset',
                         ['description']),
                        ('Extended Details',
                         [['published|4', 'updated_at']])])
    # layout = ['title|3', 'title', 'title', ['category', 'category']]

    # tabs = [
    #     ('Detail', 'articles:detail'),
    #     ('Tags', 'articles:detail-tags'),
    # ]

    def get_urls(self):
        return {
            'articles:list': (),
            'articles:detail': (self.object.pk,),
            'articles:category-detail': (self.object.category.pk,),
            'articles:detail-tags': (self.object.pk,),
        }


class ArticleCreateView(CreateView):
    page_title = _("Create Article")
    # fields = ['title', 'description', 'tags', 'category', 'published']
    model = Article
    form_class = ArticleForm
    permission_required = "add_article"
    layout = OrderedDict([
                        ('+Basic Details',
                         ['title', ['category|4', 'tags']]),
                        ('-Body|Extra Information for this fieldset',
                         ['description']),
                        ('Extended Details',
                         [['published|4', 'updated_at']])])

    def get_success_url(self):
        return reverse('articles:detail', args=(self.object.pk,))


class ArticleDeleteView(DeleteView):
    model = Article
    success_url = reverse_lazy('articles:list')
    permission_required = "delete_article"


class CategoryListView(ListView):
    page_title = _("Categories")
    model = Category
    fields = ['name']
    field_links = {
        'name': 'articles:category-detail',
    }
    tool_links = [
        (_('Create Category'), 'articles:category-create'),
    ]
    permission_required = 'view_category'
    search_fields = ('name',)


class CategoryArticlesListView(ArticleListView):

    def dispatch(self, request, *args, **kwargs):
        self.pk = kwargs.pop('pk')
        return super(CategoryArticlesListView, self).dispatch(request, *args,
                                                              **kwargs)

    # disable some settings from the default article list
    tool_links = [
    ]
    page_title = "Edit Category: Articles"
    breadcrumbs = None

    tabs = [
        ('Detail', 'articles:category-detail'),
        ('Related Articles', 'articles:category-articles-list'),
    ]
    permission_required = 'view_category'

    def get_urls(self):
        return {
            'articles:category-detail': (self.pk,),
            'articles:category-articles-list': (self.pk,),
        }

    def get_queryset(self):
        qs = super(CategoryArticlesListView, self).get_queryset()
        return qs.filter(category_id=self.pk)


class CategoryUpdateView(UpdateView):
    page_title = _("Edit Category")
    model = Category
    fields = '__all__'
    success_url = reverse_lazy('articles:category-list')
    tabs = [
        ('Detail', 'articles:category-detail'),
        ('Related Articles', 'articles:category-articles-list'),
    ]
    permission_required = 'change_category'

    def get_urls(self):
        return {
            'articles:category-detail': (self.object.pk,),
            'articles:category-articles-list': (self.object.pk,),
        }


class CategoryCreateView(CreateView):
    page_title = _("Create Category")
    model = Category
    fields = ['name']
    permission_required = 'add_category'

    def get_success_url(self):
        return reverse('articles:category-detail', args=(self.object.pk,))


class CategoryDeleteView(DeleteView):
    model = Category
    success_url = reverse_lazy('articles:category-list')
    permission_required = 'delete_category'


class TagListView(ListView):
    page_title = _("Tags")
    model = Tag
    fields = ['term']
    field_links = {
        'term': 'articles:tag-detail',
    }
    tool_links = [
        (_('Create Tag'), 'articles:tag-create'),
    ]
    permission_required = 'view_tag'


class TagUpdateView(UpdateView):
    page_title = _("Edit Tag")
    model = Tag
    fields = '__all__'
    success_url = reverse_lazy('articles:tag-list')
    permission_required = 'change_tag'


class TagCreateView(CreateView):
    page_title = _("Create Tag")
    model = Tag
    fields = ['term']
    permission_required = 'add_tag'

    def get_success_url(self):
        return reverse('articles:tag-detail', args=(self.object.pk,))


class TagDeleteView(DeleteView):
    model = Tag
    success_url = reverse_lazy('articles:tag-list')
    permission_required = 'delete_tag'



from django.db.models import Q
from arctic.forms import SimpleSearchForm


def test_simple_search_form():
    data = {'search': None}
    search_fields = ['name']

    form = SimpleSearchForm(search_fields=search_fields, data=data)
    search_filter = form.get_search_filter()

    # Empty search value, should return empty filter
    assert str(search_filter) == str(Q())

    data = {'search': 'test'}
    form = SimpleSearchForm(search_fields=search_fields, data=data)
    search_filter = form.get_search_filter()

    # Filled in search value, should return icontains filter
    assert str(search_filter) == str(Q(('name__icontains', 'test')))



from django.conf.urls import url

from articles.views import (ArticleCreateView, ArticleDeleteView,
                            ArticleListView, ArticleUpdateView,
                            CategoryArticlesListView, CategoryCreateView,
                            CategoryDeleteView, CategoryListView,
                            CategoryUpdateView, TagCreateView, TagDeleteView,
                            TagListView, TagUpdateView)

urlpatterns = [
    url(r'^$', ArticleListView.as_view(), name='list'),
    url(r'^create/$', ArticleCreateView.as_view(), name='create'),
    url(r'^(?P<pk>\d+)/$', ArticleUpdateView.as_view(), name='detail'),
    url(r'^(?P<pk>\d+)/delete/$', ArticleDeleteView.as_view(), name='delete'),
    url(r'^category/$', CategoryListView.as_view(), name='category-list'),
    url(r'^category/create/$', CategoryCreateView.as_view(),
        name='category-create'),
    url(r'^category/(?P<pk>\d+)/$', CategoryUpdateView.as_view(),
        name='category-detail'),
    url(r'^category/articles/(?P<pk>\d+)/$',
        CategoryArticlesListView.as_view(), name='category-articles-list'),
    url(r'^category/(?P<pk>\d+)/delete/$', CategoryDeleteView.as_view(),
        name='category-delete'),
    url(r'^tags/$', TagListView.as_view(), name='tag-list'),
    url(r'^tags/create/$', TagCreateView.as_view(), name='tag-create'),
    url(r'^tags/(?P<pk>\d+)/$', TagUpdateView.as_view(), name='tag-detail'),
    url(r'^tags/(?P<pk>\d+)/delete/$', TagDeleteView.as_view(),
        name='tag-delete'),
]



from __future__ import (absolute_import, unicode_literals)

import json
import os
from six.moves import urllib

from django.http import JsonResponse
from django.utils.safestring import mark_safe
from django.views.generic import TemplateView

from arctic.generics import DataListView
from arctic.utils import RemoteDataSet, offset_limit


class CountryAPIView(TemplateView):
    """
    Mock API used to demonstrate the usage of DataListView
    It's a simple implementation with support for sorting, pagination,
    filtering and field selection.
    """
    def render_to_response(self, context, **response_kwargs):
        data = []
        dir_path = os.path.dirname(os.path.realpath(__file__))
        with open(os.path.join(dir_path, 'countries.json'), 'r') as f:
            data = json.loads(f.read())

        if data:
            all_fields = data[0].keys()

        for field in all_fields:
            value = self.request.GET.get(field, None)
            if value:
                data = list(filter(lambda d: value.lower() in d[field].lower(),
                            data))

        if self.request.GET.get('fields'):
            fields = self.request.GET.get('fields').split(',')
            fields = set(all_fields).intersection(fields)
            temp = []
            for row in data:
                temp_row = {}
                for field in fields:
                    temp_row[field] = row[field]
                temp.append(temp_row)
            data = temp

        if self.request.GET.get('order_by'):
            order_by = self.request.GET.get('order_by')
            reverse = False
            if order_by[0] == '-':
                reverse = True
                order_by = order_by[1:]

            data = sorted(data, key=lambda k: k[order_by], reverse=reverse)

        if self.request.GET.get('offset') or self.request.GET.get('limit'):
            offset = int(self.request.GET.get('offset', 0))
            limit = min(int(self.request.GET.get('limit', len(data) - offset)),
                        len(data) - offset)
            if offset > len(data) or (offset + limit) > len(data):
                data = []
            else:
                data = data[offset:limit + offset]

        return JsonResponse(data, safe=False, **response_kwargs)


class CountriesDataSet(RemoteDataSet):
    url_template = 'countries-api/?{filters}{fields}{order}{paginate}'
    order_template = '&order_by={}'

    @offset_limit
    def get(self, offset, limit):
        r = urllib.request.urlopen((self.get_url(offset, limit)))
        data = r.read().decode("utf-8")
        return json.loads(data)


class CountryListView(DataListView):
    paginate_by = 10
    dataset = CountriesDataSet()
    fields = ['name', 'capital', 'flag']
    ordering_fields = ['name', 'capital']
    search_fields = ['name']
    breadcrumbs = (('Home', 'index'), ('Country List', None))
    page_title = 'Countries'
    permission_required = 'country_view'

    def get_context_data(self, **kwargs):
        # small hack to setup the current host in the url of the local API
        if not self.dataset.url_template.startswith('http'):
            self.dataset.url_template = '{}://{}/{}'.format(
                self.request.scheme,
                self.request.get_host(),
                self.dataset.url_template)
        context = super(CountryListView, self).get_context_data(**kwargs)
        return context

    def get_flag_field(self, obj):
        return mark_safe('<img style="max-width: 2rem" src="{}" />'
                         .format(obj['flag']))



# options are 'float-label', 'stacked' or 'tabular'
ARCTIC_FORM_DISPLAY = 'float-label'

ARCTIC_SITE_LOGO = 'arctic/dist/assets/img/arctic_logo.svg'

ARCTIC_SITE_NAME = 'Arctic Site Name'

ARCTIC_WIDGET_OVERLOADS = {
    'DateInput': 'arctic.widgets.DatePickerInput',
    'DateTimeInput': 'arctic.widgets.DateTimePickerInput',
    'TimeInput': 'arctic.widgets.TimePickerInput',
    'Select': 'arctic.widgets.StyledSelect',
    'SelectMultiple': 'arctic.widgets.SelectizeMultiple',
    'MultipleChoiceField': 'arctic.widgets.Selectize',
}



# TODO: Move extra views into our own views. So we can override where needed
from extra_views import InlineFormSet

from .models import Article


class ArticleInline(InlineFormSet):
    model = Article
    extra = 0
    fields = "__all__"


class TagsInline(InlineFormSet):
    model = Article.tags.through
    extra = 3
    fields = "__all__"



import os
import shutil
import sys
from setuptools import (find_packages, setup)

__VERSION__ = '1.0.0b1'


def read_md(f):
    try:
        from pypandoc import convert
        return convert(f, 'rst')
    except ImportError:
        return open(f, 'r').read()


try:
    REQUIREMENTS = open('requirements/base.txt').read()
except:
    REQUIREMENTS = None

if sys.argv[-1] == 'publish':
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    shutil.rmtree('build')
    shutil.rmtree('dist')
    sys.exit()


setup(
    name='django-arctic',
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    version=__VERSION__,
    long_description=read_md('README.md'),
    install_requires=REQUIREMENTS,
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'arctic = arctic.bin.arctic:main',
        ]},
    license='MIT',
    url='https://github.com/sanoma/django-arctic',
    author='Sanoma Netherlands',
    author_email='opensource@sanoma.com',
    description="Django framework to create custom content management systems",
)



#!/usr/bin/env python
from __future__ import absolute_import, print_function, unicode_literals

import os
import stat
from optparse import OptionParser

from django.core.management import ManagementUtility


def create_project(parser, options, args):
    # Validate args
    if len(args) < 2:
        parser.error('Please specify a name for your Arctic installation')
    elif len(args) > 3:
        parser.error('Too many arguments')

    project_name = args[1]
    try:
        dest_dir = args[2]
    except IndexError:
        dest_dir = ''

    # Make sure given name is not already in use by another
    # python package/module.
    try:
        __import__(project_name)
    except ImportError:
        pass
    else:
        parser.error('"{}" conflicts with the name of an existing '
                     'Python module and cannot be used as a project '
                     'name. Please try another name.'.format(project_name))

    print('Creating an Arctic project named {}'.format(project_name))

    # Create the project from the Arctic template using startapp

    # First find the path to Arctic
    import arctic
    arctic_path = os.path.dirname(arctic.__file__)
    template_path = os.path.join(arctic_path, 'project_template')

    # Call django-admin startproject
    utility_args = ['django-admin.py',
                    'startproject',
                    '--template=' + template_path,
                    '--ext=html,rst',
                    project_name]

    if dest_dir:
        utility_args.append(dest_dir)

    utility = ManagementUtility(utility_args)
    utility.execute()

    # add execute permission to manage.py, somehow it gets lost on the way
    manage_py = os.path.join(dest_dir or project_name, 'manage.py')
    st = os.stat(manage_py)
    os.chmod(manage_py,
             st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)

    print('Congratulations! {0} has been created.\n'
          'The next steps are:\n'
          '- In config/settings.py change the database settings (if needed).\n'
          '- Run database migrations: {0}/manage.py migrate.\n'
          '- Create an admin user: {0}/manage.py createsuperuser.\n'
          '- Finally run the project: {0}/manage.py runserver.\n'
          .format(project_name))


COMMANDS = {
    'start': create_project,
}


def main():
    # Parse options
    parser = OptionParser(usage='Usage: arctic start project_name [directory]')
    (options, args) = parser.parse_args()

    # Find command
    try:
        command = args[0]
    except IndexError:
        parser.print_help()
        return

    if command in COMMANDS:
        COMMANDS[command](parser, options, args)
    else:
        parser.error('Unrecognised command: ' + command)


if __name__ == '__main__':
    main()



from django.conf.urls import (include, url)

from arctic.generics import LoginView
from arctic.views import (handler400, handler403, handler404, handler500)  # noqa


urlpatterns = [
    url(r'^login/$', LoginView.as_view(), name='login'),
    url(r'^', include('dashboard.urls')),
]
